@@ -607,27 +607,28 @@
 
 ;; add enum symbols to dict as @code{(fixed "1")}
 (define* (udict-add-enums udict #:optional (ddict '()))
-  (define (gen-nvl edef-list ddict)
+  (define (gen-nvl edef-list udict ddict)
     (let ((def-list (and=> (canize-enum-def-list edef-list udict ddict) cdr)))
-      (fold-right
+      (fold
        (lambda (edef dd)
 	 (let ((name (sx-ref* edef 1 1)))
 	   (if (member name dd) dd (acons name (sx-ref* edef 2) dd))))
-       ddict def-list)))
-  (fold-right
-   (lambda (pair udict)
-     (if (or (pair? (car pair)) (positive? (string-length (car pair))))
-	 (let* ((specs (sx-ref (cdr pair) 1))
-		(tspec (sx-ref (sx-find 'type-spec specs) 1)))
-	   (sx-match tspec
-	     ((enum-def (ident ,name) (enum-def-list . ,defs))
-	      (gen-nvl (sx-ref tspec 2) udict))
-	     ((enum-def (enum-def-list . ,defs))
-	      (gen-nvl (sx-ref tspec 1) udict))
-	     (,_
-	      udict)))
-	 udict))
-   udict udict))
+       udict def-list)))
+  (reverse
+   (fold
+    (lambda (pair udict)
+      (if (or (pair? (car pair)) (positive? (string-length (car pair))))
+	  (let* ((specs (sx-ref (cdr pair) 1))
+		 (tspec (sx-ref (sx-find 'type-spec specs) 1)))
+	    (sx-match tspec
+	      ((enum-def (ident ,name) (enum-def-list . ,defs))
+	       (gen-nvl (sx-ref tspec 2) udict ddict))
+	      ((enum-def (enum-def-list . ,defs))
+	       (gen-nvl (sx-ref tspec 1) udict ddict))
+	      (,_
+	       udict)))
+	  udict))
+    udict udict)))
 
 
 ;; === enum handling ===================
